%module(docstring="The rumd module gives access to the C++ classes that make up the bulk of rumd software: Sample, SimulationBox, MoleculeData, potentials, integrators, output managers, etc") rumd



%init %{
  // Initialize the GPU device as part of module initialization
  int argc = 0;
  char* argv[] = {};
  try {
    Device::GetDevice().Init(argc, argv);
  }
  catch (RUMD_Error e){
    std::cerr << "RUMD_Error thrown from function " << e.className << "::" << e.methodName << ": " << std::endl << e.errorStr << std::endl;
    throw;
  }
  // Initialize the numpy module so we can use its C API (otherwise get seg 
  // faults)
  import_array();
  %}

enum SortingScheme {SORT_X, SORT_XY, SORT_XYZ};

%{
#include "rumd/Device.h" // curiously, leaving this out generates not a compile-error but a linker error....
#include "numpy/arrayobject.h"
  //#include "rumd/GPU_Timer.h"
%}



// the following allows exceptions to be passed to Python
%include exception.i
%{
#include "rumd/RUMD_Error.h"
  %}


%exception {
  try {
    $action
      }  catch (RUMD_cudaResourceError e) {
    SWIG_exception(SWIG_ValueError, e.GetCombinedErrorStringAlt().c_str());
  }
  catch (RUMD_Error e) {
    SWIG_exception(SWIG_RuntimeError,e.GetCombinedErrorString().c_str());
  } catch(...) {
    SWIG_exception(SWIG_RuntimeError, "Unknown error");
  }
 }




%feature("kwargs"); // enable keyword arguments

%feature("autodoc","1");

%include "stringTypeMap.i"
%include "vectorTypeMap.i"

%include "SimulationBox.i"

%include "Sample.i"

%include "Potentials.i"

%include "Integrators.i"

%include "IO.i"

%include "ExternalCalculators.i"

%include "MoleculeData.i"

%include "UserFunctions.i"


 // the following allows us to access the device name from Python using rumd.Device.GetDeviceName() or (for versions of python < 2.2) rumd.Device_GetDeviceName()
%nodefaultctor Device; // disable generation of wrapper for default constructor

class Device {
 public:
  static const std::string GetDeviceName();
 };


%inline %{
  std::string GetVersion() {
    return std::string(VERSION);
  }
  std::string Get_SVN_Revision() {    
    return std::string(SVN_REV);
  }
%}
