#!/usr/bin/env python

import unittest
import os
import sys
sys.path.insert(0,"../../.Swig")
sys.path.insert(0,"../../Python")

import rumd
from rumdSimulation import rumdSimulation
from pyatooms.adapters.rumd import Simulation, System, Trajectory

xyz = """\
     3
ioformat=1 dt=0.005000000 boxLengths=6.34960421,6.34960421,6.34960421 numTypes=1 Nose-Hoover-Ps=-0.027281716 Barostat-Pv=0.000000000 mass=1.000000000 columns=type,x,y,z
0       -3.159757      3.145206     -3.145651
0       -2.986284      3.045374     -2.362755
0       -2.813011      2.380848     -1.037014
"""

class TestAdaptersRUMD(unittest.TestCase):

    def setUp(self):
        self.fout = '/tmp/test_adapter_rumd_out.xyz.gz'
        self.dout = '/tmp/test_adapter_rumd_out'
        self.finp = '/tmp/test_adapter_rumd_in.xyz'
        fh = open(self.finp, 'w')
        fh.write(xyz)
        fh.close()

        self.s = rumdSimulation(self.finp, verbose=False)
        self.s.sample.SetOutputDirectory(self.dout)
        self.s.suppressAllOutput = True
        p = rumd.Pot_LJ_12_6(cutoff_method = rumd.ShiftedPotential)
        p.SetVerbose(False)
        p.SetParams(0, 0, 1., 1., 2.5)
        self.s.SetPotential(p)
        i = rumd.IntegratorNVT(targetTemperature=2.0, timeStep=0.002)
        self.s.SetIntegrator(i)

    def test_system(self):
        system = System(self.s)
        U = system.potential_energy()
        T = system.temperature()
        Uref = 36.9236726612
        self.assertLess(abs((U-Uref)/Uref), 0.001)

    def test_trajectory(self):
        t = Trajectory(self.fout, 'w')
        system = System(self.s)
        t.write_sample(system, 0, 0)
        t.close()
        #os.remove(self.f)

    def test_simulation(self):
        t = Trajectory(self.finp, 'w')
        simulation = Simulation(self.s, t)

        simulation.verbosity = 0
        self.assertEqual(simulation.verbosity, 0)
        simulation.verbosity = 1
        self.assertEqual(simulation.verbosity, 1)

        system = System(self.s)
        simulation.system = system
        # TODO: add this check to adapter
        self.s.SetBlockSize(1)
        simulation.run(1)

    def tearDown(self):
        import shutil
        if os.path.exists(self.finp):
            os.remove(self.finp)
        if os.path.exists(self.fout):
            os.remove(self.fout)
        if os.path.exists(self.dout):
            shutil.rmtree(self.dout)

if __name__ == '__main__':
    unittest.main()


