#include "rumd_trajectory.h"
#include <cassert>
#include <iostream>

class rumd_msd : public rumd_trajectory {
public:
  rumd_msd();
  ~rumd_msd() {}
  void ComputeAll(unsigned int first_block=0, int last_block=-1, unsigned int particlesPerMol = 1);

  void SetExtraTimesWithinBlock(bool set_etwb) {extraTimesWithinBlock = set_etwb; }

  void Copy_MSD_To_Array(double (*msd_array)[2], unsigned int type);
  void Copy_ISF_To_Array(double (*isf_array)[2], unsigned int type);
  void Copy_Alpha2_To_Array(double (*alpha2_array)[2], unsigned int type);

  void WriteMSD(const std::string& filename);
  void WriteMSD_CM(const std::string& filename);
  void WriteAlpha2(const std::string& filename);
  void WriteISF(const std::string& filename);
  void WriteISF_CM(const std::string& filename);
  void WriteISF_SQ(const std::string& filename);
  
  unsigned int GetNumberOfTimes() const { return Count.size(); }

  private:
  rumd_msd(const rumd_msd&);
  rumd_msd& operator=(const rumd_msd&);

  void ReadQValues();
  void ResetArrays();
  void EnsureMapKeyPresent(unsigned long rel_index);
  void CalcR2(Conf &C0, Conf &C1);
  void CalcR2cm(Conf &C0, Conf &C1, unsigned long rel_time_index);
  
  // Setup arrays to collect statistics
  std::map<unsigned long, unsigned long> Count;
  std::map<unsigned long, std::vector<double> > R2;
  std::map<unsigned long, std::vector<double> > R4;
  std::map<unsigned long, std::vector<double> > Fs;
  std::map<unsigned long, std::vector<double> > Fs2;
  std::map<unsigned long, std::vector<double> > R2cm;
  std::map<unsigned long, std::vector<double> > R4cm;
  std::map<unsigned long, std::vector<double> > Fscm;

  std::vector<double>Fs_;
  std::vector<double> qvalues;
  std::vector<int> numParticlesOfType;

  bool extraTimesWithinBlock;
  
  float dt;
};
